/*
 * Zorin Panel: Various settings and changes for the panel in Zorin Desktop..
 * Copyright (C) 2016 Zorin OS Technologies Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

const Lang = imports.lang;
const Main = imports.ui.main;
const Me = imports.misc.extensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;
const PositionHeight = Me.imports.positionHeight;
const MoveClock = Me.imports.moveClock;
const ShowDesktopButton = Me.imports.showDesktopButton;
const ActivitiesButton = Me.imports.activitiesButton;
const HotCorner = Me.imports.hotCorner;
const DynamicTransparency = Me.imports.dynamicTransparency;
const Intellihide = Me.imports.intellihide.main;
const RemoveArrows = Me.imports.removeArrows;

let settings;
let positionHeight;
let moveClock;
let showDesktopButton;
let activitiesButton;
let hotCorner;
let removeArrows;
let intellihide;
let dynamicTransparency;

function enable() {
    settings = Convenience.getSettings('org.gnome.shell.extensions.zorin-panel');

    positionHeight = new PositionHeight.zorinPanelPositionHeight(settings);
    positionHeight.enable();

    moveClock = new MoveClock.zorinPanelMoveClock(settings);
    moveClock.enable();

    showDesktopButton = new ShowDesktopButton.zorinPanelShowDesktopButton(settings);
    showDesktopButton.enable();

    activitiesButton = new ActivitiesButton.zorinPanelActivitiesButton(settings);
    activitiesButton.enable();

    hotCorner = new HotCorner.zorinPanelHotCorner(settings);
    hotCorner.enable();

    removeArrows = new RemoveArrows.zorinPanelRemoveArrows();
    removeArrows.enable();

    intellihide = new Intellihide.zorinPanelIntellihideMain(settings);
    intellihide.enable();

    // Gnome Shell 3.26 has its own implementation of dynamic transparency, so only
    // enable this extension's implementation if Gnome Shell doesn't already provide it
    if (!Main.panel._updateSolidStyle) {
        dynamicTransparency = new DynamicTransparency.zorinPanelDynamicTransparency(settings);
        dynamicTransparency.enable();
    }
}

function disable() {
    if (!Main.panel._updateSolidStyle) {
        dynamicTransparency.disable();
    }
    intellihide.disable();
    removeArrows.disable();
    hotCorner.disable();
    moveClock.disable();
    showDesktopButton.disable();
    activitiesButton.disable();
    positionHeight.disable();

    dynamicTransparency = null;
    intellihide = null;
    removeArrows = null;
    hotCorner = null;
    activitiesButton = null;
    showDesktopButton = null;
    moveClock = null;
    positionHeight = null;
    settings = null;
}
