/*
 * This code is taken from https://github.com/mlutfy/hidetopbar
 * (as at commit a2561b39d4a8e0b4438e94b18411cf8bf1889bb7)
 *
 * Licensing information:
 * Hide Top Bar extension is distributed under the terms of the
 * GNU General Public License, version 2 or later.
 */

const Main = imports.ui.main;
const Lang = imports.lang;
const MeIntellihide = imports.misc.extensionUtils.getCurrentExtension().imports.intellihide;
const MainPanelVisibilityManager = MeIntellihide.mainPanelVisibilityManager;
const WindowListPanelVisibilityManager = MeIntellihide.windowListPanelVisibilityManager;

var zorinPanelIntellihideMain = new Lang.Class({
    Name: 'ZorinPanel.IntellihideMain',

    _init: function(settings) {
        this._zorinPanelSettings = settings;
        this._bindSettingsChanges();
    },

    enable: function() {
        if (this._zorinPanelSettings.get_boolean('intellihide')) {
            this._activate_intellihide();

            if (Main.windowList)
                this._activate_intellihide_window_list();
        } else {
            this._deactivate_intellihide();
        }
    },

    enableWindowList: function() {
        if (this._zorinPanelSettings.get_boolean('intellihide')) {
            this._activate_intellihide_window_list();
        } else {
            this._deactivate_intellihide_window_list();
        }
    },

    disable: function() {
        this._deactivate_intellihide();
    },

    disableWindowList: function() {
        this._deactivate_intellihide_window_list();
    },

    _bindSettingsChanges: function() {
        this._zorinPanelSettings.connect('changed::intellihide', Lang.bind(this, this.enable));
    },

    _activate_intellihide: function() {
        this.monitorIndex = Main.layoutManager.primaryIndex;
        this.mainPVManager = new MainPanelVisibilityManager.PanelVisibilityManager(this.monitorIndex);
    },

    _activate_intellihide_window_list: function() {
        this.windowListPVManager = new WindowListPanelVisibilityManager.PanelVisibilityManager(this.monitorIndex);
    },

    _deactivate_intellihide: function() {
        if (this.mainPVManager) {
            this.mainPVManager.destroy();
            this.mainPVManager = null;
            this.monitorIndex = null;
        }

        this._deactivate_intellihide_window_list();
    },

    _deactivate_intellihide_window_list: function() {
        if (this.windowListPVManager) {
            this.windowListPVManager.destroy();
            this.windowListPVManager = null;
            this.monitorIndex = null;
        }
    }
});
