/*
 * Zorin Panel: Various settings and changes for the panel in Zorin Desktop.
 * Copyright (C) 2016-2018 Zorin OS Technologies Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 * Credits:
 * This file is based on code from the Frippery Move Clock Extension version
 * 3.26.2 by Ron Yorston. This extension can be found at the following URL:
 * http://frippery.org/extensions/
 */

const Main = imports.ui.main;
const SessionMode = imports.ui.sessionMode;
const Lang = imports.lang;

var zorinPanelMoveClock = new Lang.Class({
    Name: 'ZorinPanel.MoveClock',

    _init: function(settings) {
        this._zorinPanelSettings = settings;
    },

    enable: function() {
        this._bindSettingsChanges();

        if (this._zorinPanelSettings.get_boolean('clock-in-sys-tray')) {
            this._moveToTray();
        }
    },

    disable: function() {
       this._moveToCenter();
    },

    _bindSettingsChanges: function() {
        this._zorinPanelSettings.connect('changed::clock-in-sys-tray', Lang.bind(this, function() {
            if (this._zorinPanelSettings.get_boolean('clock-in-sys-tray')) {
               this._moveToTray();
            } else {
               this._moveToCenter();
            }
        }));
    },

    _moveToTray: function() {
        // do nothing if the clock isn't centred in this mode
        if ( Main.sessionMode.panel.center.indexOf('dateMenu') == -1 ) {
            return;
        }

        let centerBox = Main.panel._centerBox;
        let rightBox = Main.panel._rightBox;
        let dateMenu = Main.panel.statusArea['dateMenu'];
        let children = centerBox.get_children();

        // only move the clock if it's in the centre box
        if ( children.indexOf(dateMenu.container) != -1 ) {
            centerBox.remove_actor(dateMenu.container);

            children = rightBox.get_children();
            rightBox.insert_child_at_index(dateMenu.container, children.length);
       }
    },

    _moveToCenter: function() {
        // do nothing if the clock isn't centred in this mode
        if ( Main.sessionMode.panel.center.indexOf('dateMenu') == -1 ) {
            return;
        }

        let centerBox = Main.panel._centerBox;
        let rightBox = Main.panel._rightBox;
        let dateMenu = Main.panel.statusArea['dateMenu'];
        let children = rightBox.get_children();

        // only move the clock back if it's in the right box
        if ( children.indexOf(dateMenu.container) != -1 ) {
            rightBox.remove_actor(dateMenu.container);
            centerBox.add_actor(dateMenu.container);
        }
    }
});
