/*
 * This file is based on code from the Dash to Panel extension by Jason
 * DeRose (as at commit a460d2828ac38b27b7688115910f6eb6b9f274a3).
 * 
 * Code to re-anchor the panel was taken from Thoma5 BottomPanel:
 * https://github.com/Thoma5/gnome-shell-extension-bottompanel
 */

const Me = imports.misc.extensionUtils.getCurrentExtension();
const Clutter = imports.gi.Clutter;
const Gtk = imports.gi.Gtk;
const Convenience = Me.imports.convenience;
const Lang = imports.lang;
const Main = imports.ui.main;
const St = imports.gi.St;
const Meta = imports.gi.Meta;
const DND = imports.ui.dnd;
const PopupMenu = imports.ui.popupMenu;
const Tweener = imports.ui.tweener;

var zorinPanelPositionHeight = new Lang.Class({
    Name: 'ZorinPanel.PositionHeight',

    _init: function(settings) {
        this._zorinPanelSettings = settings;
    },

    enable : function() {
        this.panel = Main.panel;
        this.panelBox = Main.layoutManager.panelBox;

        this._oldPopupOpen = PopupMenu.PopupMenu.prototype.open;
        PopupMenu.PopupMenu.prototype.open = newPopupOpen;

        this._oldPopupSubMenuOpen = PopupMenu.PopupSubMenu.prototype.open;
        PopupMenu.PopupSubMenu.prototype.open = newPopupSubMenuOpen;

        this._oldUpdatePanelBarrier = Main.layoutManager._updatePanelBarrier;
        Main.layoutManager._updatePanelBarrier = Lang.bind(Main.layoutManager, newUpdatePanelBarrier);
        Main.layoutManager._updatePanelBarrier();

        this._oldPanelHeight = this.panel.actor.get_height();

        // The overview uses the this.panel height as a margin by way of a "ghost" transparent Clone
        // This pushes everything down, which isn't desired when the this.panel is moved to the bottom
        // I'm adding a 2nd ghost this.panel and will resize the top or bottom ghost depending on the this.panel position
        this._myPanelGhost = new St.Bin({ 
            child: new Clutter.Clone({ source: Main.overview._panelGhost.get_child() }),
            reactive: false,
            opacity: 0 
        });
        Main.overview._overview.add_actor(this._myPanelGhost);
        
        this._setPanelPosition();
        this._MonitorsChangedListener = global.screen.connect("monitors-changed", Lang.bind(this, function(){
            this._setPanelPosition();
        }));
        this._HeightNotifyListener = this.panelBox.connect("notify::height", Lang.bind(this, function(){
            this._setPanelPosition();
        }));

        // this is to catch changes to the window scale factor
        this._ScaleFactorListener = St.ThemeContext.get_for_stage(global.stage).connect("changed", Lang.bind(this, function () { 
            this._setPanelPosition();
        }));

        this._bindSettingsChanges();
        
        this.panel.handleDragOver = Lang.bind(this.panel, function(source, actor, x, y, time) {
            if (source == Main.xdndHandler) {
                
                // open overview so they can choose a window for focusing
                // and ultimately dropping dragged item onto
                if(Main.overview.shouldToggleByCornerOrButton())
                    Main.overview.show();
            }
            
            return DND.DragMotionResult.CONTINUE;
        });

        // Dynamic transparency is available on Gnome 3.26
        if (this.panel._updateSolidStyle) {
            this._injectionsHandler = new Convenience.InjectionsHandler();
            this.panel._zorinPanelBottom = this._zorinPanelSettings.get_boolean('panel-bottom');
            this._injectionsHandler.addWithLabel('transparency', [
                this.panel,
                '_updateSolidStyle',
                Lang.bind(this.panel, this._zorinPanelUpdateSolidStyle)
            ]);

            this.panel._updateSolidStyle();
        }
    },

    enableWindowList: function () {
        this._WindowListHeightNotifyListener = Main.windowList.actor.connect("notify::height", Lang.bind(this, function (){
            this._setPanelPosition();
        }));
    },

    disable: function () {
        if(this._WindowListHeightNotifyListener !== null && Main.windowList) {
            this.disableWindowList();
        }
        // remove this.panel styling
        if(this._HeightNotifyListener !== null) {
            this.panelBox.disconnect(this._HeightNotifyListener);
        }
        if(this._MonitorsChangedListener !== null) {
            global.screen.disconnect(this._MonitorsChangedListener);
        }
        if(this._ScaleFactorListener !== null) {
            St.ThemeContext.get_for_stage(global.stage).disconnect(this._ScaleFactorListener);
        }
        this.panel.actor.set_height(this._oldPanelHeight);
        this.panelBox.set_anchor_point(0, 0);
        Main.overview._overview.remove_child(this._myPanelGhost);
        Main.overview._panelGhost.set_height(this._oldPanelHeight);

        if (this.panel._updateSolidStyle) {
            this._injectionsHandler.removeWithLabel('transparency');
            this._injectionsHandler.destroy();
            delete this.panel._zorinPanelBottom;
        }

        PopupMenu.PopupMenu.prototype.open = this._oldPopupOpen;
        PopupMenu.PopupSubMenu.prototype.open = this._oldPopupSubMenuOpen;

        Main.layoutManager._updatePanelBarrier = this._oldUpdatePanelBarrier;
        Main.layoutManager._updatePanelBarrier();

        this._oldPanelHeight = null;
        this._ScaleFactorListener = null;
        this._MonitorsChangedListener = null;
        this._HeightNotifyListener = null;
    },

    disableWindowList: function () {
        Main.windowList.actor.disconnect(this._WindowListHeightNotifyListener);

        this._WindowListHeightNotifyListener = null;
    },

    _bindSettingsChanges: function() {
        this._zorinPanelSettings.connect('changed::panel-bottom', Lang.bind(this, function () {
            this._setPanelPosition();
        }));

        this._zorinPanelSettings.connect('changed::panel-height', Lang.bind(this, function () {
            this._setPanelPosition();
        }));
    },

    _setPanelPosition: function() {
        let scaleFactor = St.ThemeContext.get_for_stage(global.stage).scale_factor;
        let height = this._zorinPanelSettings.get_int('panel-height');
        if(scaleFactor)
            height = height*scaleFactor;

        this.panel.actor.set_height(height);

        if(Main.windowList)
            Main.windowList.actor.set_height(height);

        let isBottom = this._zorinPanelSettings.get_boolean('panel-bottom');

        Main.overview._panelGhost.set_height(isBottom ? 0 : height);
        this._myPanelGhost.set_height(isBottom ? height : 0);

        if(isBottom) {
            this.panelBox.set_anchor_point(0,(-1)*
                (Main.layoutManager.primaryMonitor.height-this.panelBox.height));

            if(Main.windowList)
                Main.windowList.actor.set_anchor_point((-1)*Main.layoutManager.primaryMonitor.x, 0);
        } else {
            this.panelBox.set_anchor_point(0, 0);

            if(Main.windowList)
                Main.windowList.actor.set_anchor_point((-1)*Main.layoutManager.primaryMonitor.x,(-1)*
                    (Main.layoutManager.primaryMonitor.height-Main.windowList.actor.height));
        }
    },

    _zorinPanelUpdateSolidStyle: function() {
        if (this.actor.has_style_pseudo_class('overview') || !Main.sessionMode.hasWindows) {
            this._removeStyleClassName('solid');
        } else {
            /* Get all the windows in the active workspace that are in the
            * primary monitor and visible */
            let activeWorkspace = global.screen.get_active_workspace();
            let windows = activeWorkspace.list_windows().filter(function(metaWindow) {
                return metaWindow.is_on_primary_monitor() &&
                    metaWindow.showing_on_its_workspace() &&
                    metaWindow.get_window_type() != Meta.WindowType.DESKTOP;
            });

            /* Check if at least one window is near enough to the panel */
            let [, panelTop] = this.actor.get_transformed_position();
            let panelBottom = panelTop + this.actor.get_height();
            let scale = St.ThemeContext.get_for_stage(global.stage).scale_factor;
            let isNearEnough = windows.some(Lang.bind(this, function(metaWindow) {
                if (this.hasOwnProperty('_zorinPanelBottom') && this._zorinPanelBottom === false) { // TODO verify this
                    let verticalPosition = metaWindow.get_frame_rect().y;
                    return verticalPosition < panelBottom + 5 * scale;
                } else {
                    let verticalPosition = metaWindow.get_frame_rect().y + metaWindow.get_frame_rect().height;
                    return verticalPosition > panelTop - 5 * scale;
                }
            }));

            if (isNearEnough)
                this._addStyleClassName('solid');
            else
                this._removeStyleClassName('solid');
        }
    }
});


function newPopupOpen(animate) {
    if (this.isOpen)
        return;

    if (this.isEmpty())
        return;

    this.isOpen = true;
    
    let side = this._boxPointer._arrowSide;
    let panelPosition = Main.layoutManager.panelBox.anchor_y == 0 ? St.Side.TOP : St.Side.BOTTOM;

    if(side != panelPosition) {
        let actor = this.sourceActor;
        while(actor) {
            if(actor == Main.panel.actor) {
                this._boxPointer._arrowSide = panelPosition;
                break;
            }
            actor = actor.get_parent();
        }
    } 
    
    this._boxPointer.setPosition(this.sourceActor, this._arrowAlignment);
    this._boxPointer.show(animate);

    this.actor.raise_top();

    this.emit('open-state-changed', true);
}


// there seems to be a rounding bug in the shell calculating the submenu size
// when the panel is at the bottom. When the height is too high to fit on the screen
// then the menu does not display at all. There's quiet a few factors involved in 
// calculating the correct height so this is a lousy hack to max it at 50% of the screen 
// height which should cover most real-world scenarios
function newPopupSubMenuOpen(animate) {
    if (this.isOpen)
        return;

    if (this.isEmpty())
        return;


    this.isOpen = true;
    this.emit('open-state-changed', true);

    let workArea = Main.layoutManager.getWorkAreaForMonitor(Main.layoutManager.primaryIndex);
    let subMenuMaxHeight = Math.floor(workArea.height / 2);
    let panelPosition = Main.layoutManager.panelBox.anchor_y == 0 ? St.Side.TOP : St.Side.BOTTOM;
    let isBottomPanelMenu = this._getTopMenu().actor.has_style_class_name('panel-menu') && panelPosition == St.Side.BOTTOM;
    
    if(isBottomPanelMenu) {
        this.actor.style = 'max-height: ' + subMenuMaxHeight + 'px;'
    }

    this.actor.show();

    let needsScrollbar = this._needsScrollbar() || (isBottomPanelMenu && this.actor.height >= subMenuMaxHeight);

    // St.ScrollView always requests space horizontally for a possible vertical
    // scrollbar if in AUTOMATIC mode. Doing better would require implementation
    // of width-for-height in St.BoxLayout and St.ScrollView. This looks bad
    // when we *don't* need it, so turn off the scrollbar when that's true.
    // Dynamic changes in whether we need it aren't handled properly.
    this.actor.vscrollbar_policy =
        needsScrollbar ? Gtk.PolicyType.AUTOMATIC : Gtk.PolicyType.NEVER;

    if (needsScrollbar)
        this.actor.add_style_pseudo_class('scrolled');
    else
        this.actor.remove_style_pseudo_class('scrolled');

    // It looks funny if we animate with a scrollbar (at what point is
    // the scrollbar added?) so just skip that case
    if (animate && needsScrollbar)
        animate = false;

    let targetAngle = this.actor.text_direction == Clutter.TextDirection.RTL ? -90 : 90;

    if (animate) {
        let [minHeight, naturalHeight] = this.actor.get_preferred_height(-1);
        this.actor.height = 0;
        this.actor._arrowRotation = this._arrow.rotation_angle_z;
        Tweener.addTween(this.actor,
                            { _arrowRotation: targetAngle,
                            height: naturalHeight,
                            time: 0.25,
                            onUpdateScope: this,
                            onUpdate: function() {
                                this._arrow.rotation_angle_z = this.actor._arrowRotation;
                            },
                            onCompleteScope: this,
                            onComplete: function() {
                                this.actor.set_height(-1);
                            }
                            });
    } else {
        this._arrow.rotation_angle_z = targetAngle;
    }
}

function newUpdatePanelBarrier() {
    if (this._rightPanelBarrier) {
        this._rightPanelBarrier.destroy();
        this._rightPanelBarrier = null;
    }
}
