/*
 * This file is based on code from the Dash to Panel extension by Jason
 * DeRose (as at commit a460d2828ac38b27b7688115910f6eb6b9f274a3).
 */

const Me = imports.misc.extensionUtils.getCurrentExtension();
const Lang = imports.lang;
const Gio = imports.gi.Gio;
const Main = imports.ui.main;
const Mainloop = imports.mainloop;
const St = imports.gi.St;
const Shell = imports.gi.Shell;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;

let tracker = Shell.WindowTracker.get_default();

const show_desktop_icon = Me.path + '/show-desktop-symbolic.svg';

var zorinPanelShowDesktopButton = new Lang.Class({
    Name: 'ZorinPanel.ShowDesktopButton',

    _init: function(settings) {
        this._zorinPanelSettings = settings;
    },

    enable : function() {
        this.panel = Main.panel;
        this._displayShowDesktopButton(this._zorinPanelSettings.get_boolean('show-showdesktop-button'));
        this._bindSettingsChanges();
    },

    disable: function () {
        this._displayShowDesktopButton(false);
    },

    _bindSettingsChanges: function() {
        this._zorinPanelSettings.connect('changed::show-showdesktop-button', Lang.bind(this, function() {
            this._displayShowDesktopButton(this._zorinPanelSettings.get_boolean('show-showdesktop-button'));
        }));
    },

    _displayShowDesktopButton: function (isVisible) {
        if(isVisible) {
            if(this._showDesktopButton)
                return;

            this._showDesktopButton = new zorinPanelDesktopButton;

            this.panel.addToStatusArea('desktop-button', this._showDesktopButton, this.panel._rightBox.get_n_children());
        } else {
            if(!this._showDesktopButton)
                return;

            this._showDesktopButton.destroy();
            this._showDesktopButton = null;
        }
    }
});

var zorinPanelDesktopButton = new Lang.Class({
    Name: 'ZorinPanel.DesktopButton',
    Extends: PanelMenu.Button,

    _init: function() {
        this.parent(null, "Desktop Button");

        let hbox = new St.BoxLayout({ style_class: 'panel-status-menu-box' });
        let icon = new St.Icon({ gicon: Gio.icon_new_for_string(show_desktop_icon),
                                 style_class: 'system-status-icon' });

        hbox.add_child(icon);
        this.actor.add_child(hbox);

        this.actor.connect('button-press-event', Lang.bind(this, this._onShowDesktopButtonPress));
    },

    _onShowDesktopButtonPress: function() {
        if(this._focusAppChangeId){
            tracker.disconnect(this._focusAppChangeId);
            this._focusAppChangeId = null;
        }

        if(this._restoreWindowList && this._restoreWindowList.length) {
            let current_workspace = global.screen.get_active_workspace();
            let windows = current_workspace.list_windows();
            this._restoreWindowList.forEach(function(w) {
                if(windows.indexOf(w) > -1)
                    Main.activateWindow(w);
            });
            this._restoreWindowList = null;

            Main.overview.hide();
        } else {
            let current_workspace = global.screen.get_active_workspace();
            let windows = current_workspace.list_windows().filter(function (w) {
                return w.showing_on_its_workspace() && !w.skip_taskbar;
            });
            windows = global.display.sort_windows_by_stacking(windows);

            windows.forEach(function(w) {
                w.minimize();
            });
            
            this._restoreWindowList = windows;

            Mainloop.timeout_add(0, Lang.bind(this, function () {
                this._focusAppChangeId = tracker.connect('notify::focus-app', Lang.bind(this, function () {
                    this._restoreWindowList = null;
                }));
            }));

            Main.overview.hide();
        }
    }
});
